import strawberry
from typing import List, Optional
from strawberry.types import Info

from siaql.graphql.schemas.types import Alert, Hash256
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput

from siaql.graphql.resolvers.hostd import HostdBaseResolver


@strawberry.type
class AlertQueries:
    @strawberry.field
    async def hostd_alerts(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[Alert]:
        """Get active alerts"""
        return await HostdBaseResolver.handle_api_call(
            info, "get_alerts", filter_input=filter, sort_input=sort, pagination_input=pagination
        )


@strawberry.type
class AlertMutations:
    @strawberry.mutation
    async def hostd_dismiss_alerts(self, info: Info, ids: List[Hash256]) -> bool:
        """Dismiss specified alerts"""
        await HostdBaseResolver.handle_api_call(info, "post_alerts_dismiss", ids=ids)
        return True



import strawberry
from typing import List
from strawberry.types import Info

from siaql.graphql.schemas.types import (
    WalletResponse,
    WalletEvent,
    Address,
    Currency,
    TransactionID,
    WalletSendSiacoinsRequest,
)
from siaql.graphql.resolvers.hostd import HostdBaseResolver
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput

from typing import Optional


@strawberry.type
class WalletQueries:
    @strawberry.field
    async def hostd_wallet(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> WalletResponse:
        """Get wallet state"""
        return await HostdBaseResolver.handle_api_call(
            info, "get_wallet", filter_input=filter, sort_input=sort, pagination_input=pagination
        )

    @strawberry.field
    async def hostd_wallet_events(
        self,
        info: Info,
        limit: int = 100,
        offset: int = 0,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[WalletEvent]:
        """Get wallet events with pagination"""
        return await HostdBaseResolver.handle_api_call(
            info,
            "get_wallet_events",
            limit=limit,
            offset=offset,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def hostd_wallet_pending(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[WalletEvent]:
        """Get pending wallet events"""
        return await HostdBaseResolver.handle_api_call(
            info, "get_wallet_pending", filter_input=filter, sort_input=sort, pagination_input=pagination
        )


@strawberry.type
class WalletMutations:
    @strawberry.mutation
    async def hostd_send_siacoins(self, info: Info, req: WalletSendSiacoinsRequest.Input) -> TransactionID:
        """Send siacoins to an address"""
        return await HostdBaseResolver.handle_api_call(info, "post_wallet_send", req=req)



import strawberry
from typing import Optional
from strawberry.types import Info

from siaql.graphql.schemas.types import ChainIndex
from siaql.graphql.resolvers.hostd import HostdBaseResolver
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput


@strawberry.type
class IndexQueries:
    @strawberry.field
    async def hostd_index_tip(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> ChainIndex:
        """Get the current index tip"""
        return await HostdBaseResolver.handle_api_call(
            info, "get_index_tip", filter_input=filter, sort_input=sort, pagination_input=pagination
        )



from datetime import datetime
from typing import Any, Dict, List, Optional

import strawberry
from strawberry.types import Info
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput

from siaql.graphql.resolvers.walletd import WalletdBaseResolver
from siaql.graphql.schemas.types import (
    SiacoinElement,
    SiafundElement,
    Wallet,
    Balance,
    Address,
)
from siaql.graphql.schemas.types import (
    WalletUpdateRequest,
    WalletReserveRequest,
    WalletReleaseRequest,
    WalletFundRequest,
    WalletEvent,
    WalletFundSFRequest,
    WalletFundResponse,
)


@strawberry.type
class WalletQueries:
    @strawberry.field
    async def walletd_wallets(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[Wallet]:
        """Get all wallets"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_wallets",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def walletd_wallet_addresses(
        self,
        info: Info,
        wallet_id: str,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[Address]:
        """Get addresses for a wallet"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_wallet_addresses",
            wallet_id=wallet_id,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def walletd_wallet_balance(
        self,
        info: Info,
        wallet_id: str,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Balance:
        """Get wallet balance"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_wallet_balance",
            wallet_id=wallet_id,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def walletd_wallet_events(
        self,
        info: Info,
        wallet_id: str,
        offset: int = 0,
        limit: int = 500,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[WalletEvent]:
        """Get wallet events"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_wallet_events",
            wallet_id=wallet_id,
            offset=offset,
            limit=limit,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def walletd_wallet_unconfirmed_events(
        self,
        info: Info,
        wallet_id: str,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[WalletEvent]:
        """Get unconfirmed wallet events"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_wallet_unconfirmed_events",
            wallet_id=wallet_id,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def walletd_wallet_siacoin_outputs(
        self,
        info: Info,
        wallet_id: str,
        offset: Optional[int] = 0,
        limit: Optional[int] = 1000,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[SiacoinElement]:
        """Get wallet siacoin outputs"""
        data = await WalletdBaseResolver.handle_api_call(
            info,
            "get_wallet_siacoin_outputs",
            wallet_id=wallet_id,
            offset=offset,
            limit=limit,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )
        return data

    @strawberry.field
    async def walletd_wallet_siafund_outputs(
        self,
        info: Info,
        wallet_id: str,
        offset: Optional[int] = 0,
        limit: Optional[int] = 1000,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[SiafundElement]:
        """Get wallet siafund outputs"""
        data = await WalletdBaseResolver.handle_api_call(
            info,
            "get_wallet_siafund_outputs",
            wallet_id=wallet_id,
            offset=offset,
            limit=limit,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )
        return data


@strawberry.type
class WalletMutations:
    @strawberry.mutation
    async def walletd_add_wallet(self, info: Info, wallet: WalletUpdateRequest.Input) -> Wallet:
        """Add a new wallet"""
        return await WalletdBaseResolver.handle_api_call(info, "post_add_wallet", wallet_update=wallet)

    @strawberry.mutation
    async def walletd_update_wallet(self, info: Info, wallet_id: str, wallet: WalletUpdateRequest.Input) -> Wallet:
        """Update a wallet"""
        return await WalletdBaseResolver.handle_api_call(
            info, "post_update_wallet", wallet_id=wallet_id, wallet_update=wallet
        )

    @strawberry.mutation
    async def walletd_delete_wallet(self, info: Info, wallet_id: str) -> bool:
        """Delete a wallet"""
        await WalletdBaseResolver.handle_api_call(info, "delete_wallet", wallet_id=wallet_id)
        return True

    @strawberry.mutation
    async def walletd_add_wallet_address(self, info: Info, wallet_id: str, address: Address) -> bool:
        """Add an address to a wallet"""
        await WalletdBaseResolver.handle_api_call(info, "add_wallet_address", wallet_id=wallet_id, address=address)
        return True

    @strawberry.mutation
    async def walletd_remove_wallet_address(self, info: Info, wallet_id: str, address: str) -> bool:
        """Remove an address from a wallet"""
        await WalletdBaseResolver.handle_api_call(info, "delete_wallet_address", wallet_id=wallet_id, address=address)
        return True

    @strawberry.mutation
    async def walletd_reserve_outputs(self, info: Info, wallet_id: str, request: WalletReserveRequest.Input) -> bool:
        """Reserve outputs"""
        await WalletdBaseResolver.handle_api_call(
            info, "post_wallet_reserve", wallet_id=wallet_id, reserve_request=request
        )
        return True

    @strawberry.mutation
    async def walletd_release_outputs(self, info: Info, wallet_id: str, request: WalletReleaseRequest.Input) -> bool:
        """Release outputs"""
        await WalletdBaseResolver.handle_api_call(
            info, "post_wallet_release", wallet_id=wallet_id, release_request=request
        )
        return True

    @strawberry.mutation
    async def walletd_fund_transaction(
        self, info: Info, wallet_id: str, request: WalletFundRequest.Input
    ) -> WalletFundResponse:
        return await WalletdBaseResolver.handle_api_call(
            info, "post_wallet_fund", wallet_id=wallet_id, fund_request=request
        )

    @strawberry.mutation
    async def walletd_fund_siafund_transaction(
        self, info: Info, wallet_id: str, request: WalletFundSFRequest.Input
    ) -> WalletFundResponse:
        """Fund a siafund transaction"""
        return await WalletdBaseResolver.handle_api_call(
            info, "post_wallet_fund_siafund", wallet_id=wallet_id, fund_request=request
        )



import logging
import sys


def setup_logging(level=logging.INFO):
    # Create logger
    logger = logging.getLogger("siaql")
    logger.setLevel(level)

    # Create console handler and set level
    console_handler = logging.StreamHandler(sys.stdout)
    console_handler.setLevel(level)

    # Create formatter
    formatter = logging.Formatter("%(levelname)s - %(asctime)s - %(name)s -  - %(message)s")

    # Add formatter to console handler
    console_handler.setFormatter(formatter)

    # Add console handler to logger
    logger.addHandler(console_handler)

    return logger



from typing import List, Optional, Union
from strawberry.types import Info
import strawberry
from datetime import datetime
from siaql.graphql.resolvers.walletd import WalletdBaseResolver
from siaql.graphql.schemas.types import WalletEvent, Hash256

from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput


@strawberry.type
class EventQueries:
    @strawberry.field
    async def walletd_event(
        self,
        info: Info,
        event_id: Hash256,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> WalletEvent:
        """Get a specific event"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_event",
            event_id=event_id,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )



# siaql/graphql/resolvers/walletd.py
import enum
import inspect
import logging
from dataclasses import fields
from datetime import datetime
from functools import wraps
from typing import (
    Any,
    Callable,
    Dict,
    List,
    Optional,
    Type,
    TypeVar,
    Union,
    get_args,
    get_origin,
    get_type_hints,
)

import strawberry
from strawberry.exceptions import MissingTypesForGenericError
from strawberry.types import Info
from strawberry.types.base import StrawberryList, StrawberryOptional, StrawberryType
from strawberry.types.enum import EnumDefinition as StrawberryEnum
from strawberry.types.lazy_type import LazyType
from strawberry.types.scalar import ScalarDefinition, ScalarWrapper
from strawberry.types.union import StrawberryUnion
from dateutil import parser


logger = logging.getLogger("siaql.resolvers.converter")


class TypeConverter:
    @staticmethod
    def parse_datetime(value: str) -> datetime:
        """
        Parse datetime strings with various formats.
        Uses python-dateutil for maximum compatibility.
        """
        if not isinstance(value, str):
            raise ValueError(f"Expected string for datetime parsing, got {type(value)}")

        try:
            # First try direct fromisoformat
            return datetime.fromisoformat(value)
        except ValueError:
            try:
                # If that fails, try with dateutil parser
                return parser.parse(value)
            except (ValueError, TypeError) as e:
                # If all parsing attempts fail
                raise ValueError(f"Unable to parse datetime string: {value}")

    @classmethod
    def get_base_type(cls, type_obj: Any) -> Type:
        """Extract the base type from Strawberry type wrappers"""
        if isinstance(type_obj, StrawberryOptional):
            return cls.get_base_type(type_obj.of_type)
        elif isinstance(type_obj, StrawberryList):
            return cls.get_base_type(type_obj.of_type)
        elif isinstance(type_obj, ScalarWrapper):
            return type_obj.wrap
        elif isinstance(type_obj, LazyType):
            return cls.get_base_type(type_obj.resolve_type())
        elif hasattr(type_obj, "of_type"):
            return cls.get_base_type(type_obj.of_type)
        return type_obj

    @classmethod
    def get_wrapped_type(cls, type_obj: Any) -> Type:
        """Get the wrapped type without unwrapping Optional"""
        if isinstance(type_obj, StrawberryList):
            return cls.get_wrapped_type(type_obj.of_type)
        elif isinstance(type_obj, ScalarWrapper):
            return type_obj
        elif isinstance(type_obj, LazyType):
            return cls.get_wrapped_type(type_obj.resolve_type())
        elif hasattr(type_obj, "of_type") and not isinstance(type_obj, StrawberryOptional):
            return cls.get_wrapped_type(type_obj.of_type)
        return type_obj

    @classmethod
    def convert_value(cls, value: Any, target_type: Type) -> Any:
        """Convert a value to the target type, handling nested structures"""
        if value is None:
            return None

        if isinstance(target_type, StrawberryOptional):
            return cls.convert_value(value, target_type.of_type)  # Unwrap optional

        # Handle StrawberryList
        if isinstance(target_type, StrawberryList):
            if value is None:
                return None

            if not isinstance(value, (list, tuple)):
                return None
            # Convert each item in the list using the list's element type
            return [cls.convert_value(item, target_type.of_type) for item in value]

        # Handle LazyType
        if isinstance(target_type, LazyType):
            return cls.convert_value(value, target_type.resolve_type())

        # Get the wrapped type (preserving Optional wrapper)
        wrapped_type = cls.get_wrapped_type(target_type)
        # Get the actual base type (removing all wrappers)
        base_type = cls.get_base_type(target_type)

        # Handle Union types
        if get_origin(base_type) is Union:
            possible_types = [t for t in get_args(base_type) if t is not type(None)]
            # Try each possible type until one works
            for possible_type in possible_types:
                try:
                    return cls.convert_value(value, possible_type)
                except (ValueError, TypeError):
                    continue
            return value

        # Handle Strawberry Enums
        if isinstance(wrapped_type, StrawberryEnum) or (
            inspect.isclass(base_type) and issubclass(base_type, enum.Enum)
        ):
            if isinstance(value, str):
                try:
                    return base_type[value.upper()]
                except KeyError:
                    # Try by value if name lookup fails
                    return next((member for member in base_type if member.value == value), value)
            if isinstance(value, (int, float)):
                try:
                    return base_type(value)
                except ValueError:
                    return value
            return value

        # Handle Strawberry Scalars
        if isinstance(wrapped_type, ScalarWrapper):
            if hasattr(wrapped_type, "parse_value"):
                return wrapped_type.parse_value(value)
            return base_type(value)

        # Handle SiaType subclasses and other Strawberry types
        if isinstance(value, dict) and (
            hasattr(base_type, "__strawberry_definition__")
            or (inspect.isclass(base_type) and issubclass(base_type, strawberry.type))
        ):
            converted = cls.convert_to_strawberry_type(value, base_type)
            # Create an instance if needed
            if inspect.isclass(base_type) and not isinstance(converted, base_type):
                # Get all required fields with their default values
                required_fields = cls.get_required_fields(base_type)
                # Merge converted data with required fields
                all_fields = {**required_fields, **converted}
                return base_type(**all_fields)
            return converted

        # Handle basic types
        if isinstance(base_type, type):
            try:
                if issubclass(base_type, str):
                    return str(value)
                elif issubclass(base_type, (int, float)):
                    return base_type(value)
                elif issubclass(base_type, datetime) and isinstance(value, str):
                    return cls.parse_datetime(value)
                elif issubclass(base_type, enum.Enum):
                    if isinstance(value, str):
                        try:
                            return base_type[value.upper()]
                        except KeyError:
                            return next((member for member in base_type if member.value == value), value)
                    return base_type(value)
            except TypeError:
                pass

        return value

    @classmethod
    def get_all_fields(cls, target_type: Type) -> Dict[str, Any]:
        """Get all fields including inherited ones"""
        all_fields = {}

        # Get fields from base classes
        for base in getattr(target_type, "__mro__", [])[1:]:
            if hasattr(base, "__annotations__"):
                try:
                    for field in fields(base):
                        all_fields[field.name] = field
                except TypeError:
                    continue

        # Get fields from the class itself
        try:
            for field in fields(target_type):
                all_fields[field.name] = field
        except TypeError:
            pass

        return all_fields

    @classmethod
    def get_field_name_mapping(cls, field: Any) -> tuple[str, str]:
        """Get both Python name and JSON name for a field"""
        python_name = field.python_name if hasattr(field, "python_name") else field.name
        json_name = field.graphql_name if hasattr(field, "graphql_name") else field.name
        return python_name, json_name

    @classmethod
    def convert_to_strawberry_type(cls, data: Dict[str, Any], target_type: Type) -> Any:
        """Convert a dictionary to a Strawberry type, handling nested fields"""
        if not isinstance(data, dict):
            return data

        if isinstance(target_type, ScalarWrapper):
            return target_type.parse_value(data)

        field_mappings = {}  # JSON name -> (Python name, field)
        all_fields = cls.get_all_fields(target_type)

        for field in all_fields.values():
            python_name, json_name = cls.get_field_name_mapping(field)
            field_mappings[json_name] = (python_name, field)
            if python_name != json_name:
                field_mappings[python_name] = (python_name, field)

        result = {}

        for key, value in data.items():
            if key in field_mappings:
                python_name, field = field_mappings[key]
                field_type = field.type if hasattr(field, "type") else field
                try:
                    converted_value = cls.convert_value(value, field_type)
                    result[python_name] = converted_value
                except Exception as e:
                    logger.error("Error converting field %s: %s", key, str(e))
                    result[python_name] = None
            else:
                logger.warning("No mapping found for %s", key)

        return result

    @classmethod
    def get_required_fields(cls, target_type: Type) -> Dict[str, Any]:
        """Get all fields of a type with None as default for Optional fields"""
        result = {}
        all_fields = cls.get_all_fields(target_type)

        for field in all_fields.values():
            python_name, _ = cls.get_field_name_mapping(field)
            field_type = field.type

            # Check if field is Optional
            is_optional = isinstance(field_type, StrawberryOptional) or (
                get_origin(field_type) is Union and type(None) in get_args(field_type)
            )

            # Always use python_name for the initialization
            result[python_name] = None if is_optional else getattr(field, "default", None)

        return result

    @classmethod
    def convert(cls, value: Any, target_type: Type) -> Any:
        """Main entry point for type conversion"""
        return cls.convert_value(value, target_type)



import strawberry
from strawberry.types import Info

from siaql.graphql.schemas.types import Currency
from siaql.graphql.resolvers.hostd import HostdBaseResolver
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput

from typing import Optional


@strawberry.type
class TPoolQueries:
    @strawberry.field
    async def hostd_tpool_fee(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Currency:
        """Get recommended transaction fee"""
        return await HostdBaseResolver.handle_api_call(
            info, "get_tpool_fee", filter_input=filter, sort_input=sort, pagination_input=pagination
        )



import strawberry
from typing import List, Optional
from datetime import datetime
from strawberry.types import Info

from siaql.graphql.schemas.types import Metrics, MetricsInterval
from siaql.graphql.resolvers.hostd import HostdBaseResolver
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput


@strawberry.type
class MetricsQueries:
    @strawberry.field
    async def hostd_metrics(
        self,
        info: Info,
        timestamp: Optional[datetime] = None,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Metrics:
        """Get metrics at specified timestamp"""
        return await HostdBaseResolver.handle_api_call(
            info, "get_metrics", timestamp=timestamp, filter_input=filter, sort_input=sort, pagination_input=pagination
        )

    @strawberry.field
    async def hostd_period_metrics(
        self,
        info: Info,
        start: datetime,
        periods: int,
        interval: MetricsInterval,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[Metrics]:
        """Get metrics for multiple periods"""
        return await HostdBaseResolver.handle_api_call(
            info,
            "get_period_metrics",
            start=start,
            periods=periods,
            interval=interval,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )



# siaql/siaql/cli/main.py
import typer
import uvicorn
import os
import sys
from rich.console import Console
from rich.prompt import Prompt, Confirm
from rich.table import Table
from typing import Optional
from pathlib import Path
from dotenv import load_dotenv
from siaql.graphql.app import create_graphql_app
import httpx
# Load environment variables from .env file
load_dotenv()

app = typer.Typer(help="SiaQL - GraphQL interface for Sia network components")
console = Console()


def validate_url(url: str) -> bool:
    """Check if a URL is alive by making a request"""
    try:
        with httpx.Client() as client:
            response = client.get(url)
            return response.status_code < 500  # Accept any non-server error response
    except:
        return False


def get_endpoint_config(name: str, default_url: str) -> tuple[Optional[str], Optional[str], bool]:
    """Get URL and password for an endpoint with validation"""
    while True:
        url = Prompt.ask(f"Enter {name} URL", default=default_url)

        if not validate_url(url):
            console.print(f"[yellow][bold]WARNING:[/yellow][/bold] {url} seems to be unreachable")
            if Confirm.ask("Would you like to edit the URL?", default=True):
                continue
            if Confirm.ask("Would you like to skip this endpoint?", default=False):
                return None, None, True

        password = Prompt.ask(f"Enter {name} API password", password=True)
        return url, password, False


@app.command()
def serve(
    host: str = typer.Option(None, help="Host to bind the server to", envvar="HOST"),
    port: int = typer.Option(None, help="Port to bind the server to", envvar="PORT"),
    walletd_url: str = typer.Option(None, help="Walletd API URL", envvar="WALLETD_URL"),
    walletd_password: Optional[str] = typer.Option(None, help="Walletd API password", envvar="WALLETD_PASSWORD"),
    renterd_url: str = typer.Option(None, help="Renterd API URL", envvar="RENTERD_URL"),
    renterd_password: Optional[str] = typer.Option(None, help="Renterd API password", envvar="RENTERD_PASSWORD"),
    hostd_url: str = typer.Option(None, help="Hostd API URL", envvar="HOSTD_URL"),
    hostd_password: Optional[str] = typer.Option(None, help="Hostd API password", envvar="HOSTD_PASSWORD"),
    skip_walletd: bool = typer.Option(False, help="Skip walletd configuration", envvar="SKIP_WALLETD"),
    skip_renterd: bool = typer.Option(False, help="Skip renterd configuration", envvar="SKIP_RENTERD"),
    skip_hostd: bool = typer.Option(False, help="Skip hostd configuration", envvar="SKIP_HOSTD"),
):
    """Start the GraphQL server"""

    # Set defaults
    DEFAULT_HOST = "127.0.0.1"
    DEFAULT_PORT = 9090
    DEFAULT_RENTERD_URL = "http://localhost:9981"
    DEFAULT_WALLETD_URL = "http://localhost:9982"
    DEFAULT_HOSTD_URL = "http://localhost:9983"

    # Get values from environment or use defaults
    host = host or os.getenv("HOST") or DEFAULT_HOST
    port = port or int(os.getenv("PORT", "0")) or DEFAULT_PORT

    # Track skipped endpoints
    skipped_endpoints = {"walletd": False, "renterd": False, "hostd": False}

    # Check enable flags from environment
    enable_walletd = os.getenv("ENABLE_WALLETD", "true").lower() == "true" and not skip_walletd
    enable_renterd = os.getenv("ENABLE_RENTERD", "true").lower() == "true" and not skip_renterd
    enable_hostd = os.getenv("ENABLE_HOSTD", "true").lower() == "true" and not skip_hostd

    # Handle walletd configuration
    if not enable_walletd or skip_walletd:
        skipped_endpoints["walletd"] = True
        walletd_url = None
        walletd_password = None
    elif not (walletd_url and walletd_password):
        walletd_url, walletd_password, skipped_endpoints["walletd"] = get_endpoint_config(
            "walletd", DEFAULT_WALLETD_URL
        )

    # Handle renterd configuration
    if not enable_renterd or skip_renterd:
        skipped_endpoints["renterd"] = True
        renterd_url = None
        renterd_password = None
    elif not (renterd_url and renterd_password):
        renterd_url, renterd_password, skipped_endpoints["renterd"] = get_endpoint_config(
            "renterd", DEFAULT_RENTERD_URL
        )

    # Handle hostd configuration
    if not enable_hostd or skip_hostd:
        skipped_endpoints["hostd"] = True
        hostd_url = None
        hostd_password = None
    elif not (hostd_url and hostd_password):
        hostd_url, hostd_password, skipped_endpoints["hostd"] = get_endpoint_config("hostd", DEFAULT_HOSTD_URL)

    # Check if all endpoints are skipped
    if all(skipped_endpoints.values()):
        console.print(
            "[red][bold]ERROR:[/bold] All services have been skipped. At least one service must be enabled.[/red]"
        )
        sys.exit(1)

    console.print(f"Starting [purple][bold]SiaQL[/bold][/purple] on http://{host}:{port}/graphql")
    console.print(f"GraphiQL interface available at http://{host}:{port}/graphql")
    console.print(
        f"[cyan]Enabled services: {', '.join([key for key, value in skipped_endpoints.items() if not value])}[/cyan]"
    )
    console.print(
        """[green]           
        #############        
   ***               ###     
   ***                  ##   
  ##       =======       ##  
 ##     =============     ## 
 #     =====      ====     # 
 #     ====       ====     # 
 #     =====      ====     # 
 ##     ==============    ## 
  ##      ============   ##  
   ##                   ##   
     ###             ###     
        #############       [/green]
        """
    )
    # Create and display project information table
    table = Table(title="[bold]SiaQL Project Information[/bold]", show_header=False, show_edge=False)
    table.add_column("Key", style="cyan")
    table.add_column("Value", style="white")

    table.add_row("Repository", "[link=https://github.com/justmert/siaql]github.com/justmert/siaql[/link]")
    table.add_row("Documentation", "[link=https://siaql-docs.netlify.app]siaql-docs.netlify.app[/link]")

    console.print()  # Add empty line for spacing
    console.print(table)
    console.print()  # Add empty line for spacing

    graphql_app = create_graphql_app(
        walletd_url=walletd_url,
        walletd_password=walletd_password,
        renterd_url=renterd_url,
        renterd_password=renterd_password,
        hostd_url=hostd_url,
        hostd_password=hostd_password,
        skipped_endpoints=skipped_endpoints,
    )

    uvicorn.run(graphql_app, host=host, port=port, log_level="info")


if __name__ == "__main__":
    app()



import strawberry
from typing import List
from strawberry.types import Info

from siaql.graphql.schemas.types import RegisterWebHookRequest, Webhook
from siaql.graphql.resolvers.hostd import HostdBaseResolver
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput
from typing import Optional


@strawberry.type
class WebhookQueries:
    @strawberry.field
    async def hostd_webhooks(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[Webhook]:
        """Get list of webhooks"""
        return await HostdBaseResolver.handle_api_call(
            info, "get_webhooks", filter_input=filter, sort_input=sort, pagination_input=pagination
        )


@strawberry.type
class WebhookMutations:
    @strawberry.mutation
    async def hostd_register_webhook(self, info: Info, req: RegisterWebHookRequest.Input) -> Webhook:
        """Register a new webhook"""
        return await HostdBaseResolver.handle_api_call(info, "post_webhooks", req=req)

    @strawberry.mutation
    async def hostd_update_webhook(self, info: Info, id: int, req: RegisterWebHookRequest.Input) -> Webhook:
        """Update an existing webhook"""
        return await HostdBaseResolver.handle_api_call(info, "put_webhooks", id=id, req=req)

    @strawberry.mutation
    async def hostd_delete_webhook(self, info: Info, id: int) -> bool:
        """Delete a webhook"""
        await HostdBaseResolver.handle_api_call(info, "delete_webhooks", id=id)
        return True

    @strawberry.mutation
    async def hostd_test_webhook(self, info: Info, id: int) -> bool:
        """Test a webhook"""
        await HostdBaseResolver.handle_api_call(info, "post_webhooks_test", id=id)
        return True



import os
import subprocess
from pathlib import Path


def main():
    # Get the project root directory
    project_root = Path(__file__).parent.parent.parent
    docs_config = project_root / "docs" / "config.yml"
    output_dir = project_root / "docs" / "public"

    # Create output directory if it doesn't exist
    output_dir.mkdir(parents=True, exist_ok=True)

    # Run SpectaQL with the config file
    subprocess.run(
        [
            "spectaql",
            str(docs_config),
            "--target-dir",
            str(output_dir),
        ],
        check=True,
    )

    print(f"Documentation has been built successfully in {output_dir}")


if __name__ == "__main__":
    main()



# graphql/schemas/walletd/rescan.py

import strawberry
from strawberry.types import Info
from siaql.graphql.resolvers.walletd import WalletdBaseResolver
from siaql.graphql.schemas.types import RescanResponse
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput
from typing import Optional


@strawberry.type
class RescanQueries:
    @strawberry.field
    async def walletd_rescan_status(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> RescanResponse:
        """Get rescan status"""
        data = await WalletdBaseResolver.handle_api_call(
            info,
            "get_rescan_status",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )
        return data


@strawberry.type
class RescanMutations:
    @strawberry.mutation
    async def walletd_start_rescan(self, info: Info, height: int) -> bool:
        """Start rescan from height"""
        await WalletdBaseResolver.handle_api_call(info, "start_rescan", height=height)
        return True



# siaql/siaql/graphql/app.py
from typing import Optional, Union, Dict, Any
from strawberry.asgi import GraphQL
from starlette.requests import Request
from starlette.websockets import WebSocket
from starlette.responses import Response
from siaql.graphql.schema import schema
from siaql.api.walletd import WalletdClient
from siaql.api.renterd import RenterdClient
from siaql.api.hostd import HostdClient


class SiaQLGraphQL(GraphQL):
    def __init__(
        self,
        walletd_url: str,
        walletd_password: str,
        renterd_url: str,
        renterd_password: str,
        hostd_url: str,
        hostd_password: str,
        skipped_endpoints: Dict[str, bool],
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        self.skipped_endpoints = skipped_endpoints

        # Initialize clients only for non-skipped endpoints
        self.walletd_client = (
            None if skipped_endpoints["walletd"] else WalletdClient(base_url=walletd_url, api_password=walletd_password)
        )
        self.renterd_client = (
            None if skipped_endpoints["renterd"] else RenterdClient(base_url=renterd_url, api_password=renterd_password)
        )
        self.hostd_client = (
            None if skipped_endpoints["hostd"] else HostdClient(base_url=hostd_url, api_password=hostd_password)
        )

    async def get_context(
        self, request: Union[Request, WebSocket], response: Optional[Response] = None
    ) -> Dict[str, Any]:
        """Provides the context for GraphQL resolvers"""
        context = {
            "request": request,
            "response": response,
            "walletd_client": self.walletd_client,
            "renterd_client": self.renterd_client,
            "hostd_client": self.hostd_client,
            "skipped_endpoints": self.skipped_endpoints,
        }
        return context


def create_graphql_app(
    walletd_url: str,
    walletd_password: str,
    renterd_url: str,
    renterd_password: str,
    hostd_url: str,
    hostd_password: str,
    skipped_endpoints: Dict[str, bool],
) -> GraphQL:
    """Creates and configures the GraphQL application"""
    return SiaQLGraphQL(
        schema=schema,
        walletd_url=walletd_url,
        walletd_password=walletd_password,
        renterd_url=renterd_url,
        renterd_password=renterd_password,
        hostd_url=hostd_url,
        hostd_password=hostd_password,
        skipped_endpoints=skipped_endpoints,
        graphiql=True,
        debug=True,
    )



from typing import Any, Dict, List, Optional

import strawberry
from strawberry.types import Info

from siaql.graphql.resolvers.renterd import RenterdBaseResolver

from siaql.graphql.schemas.types import (
    Account,
    AccountsFundRequest,
    AccountsFundResponse,
    AccountsSaveRequest,
    AddObjectRequest,
    AddPartialSlabResponse,
    Alert,
    AlertsOpts,
    AlertsResponse,
    ArchivedContract,
    Autopilot,
    Block,
    Bucket,
    BucketCreateRequest,
    BucketPolicy,
    BusStateResponse,
    ConsensusState,
    ContractAcquireRequest,
    ContractAcquireResponse,
    ContractAddRequest,
    ContractFormRequest,
    ContractKeepaliveRequest,
    ContractMetadata,
    ContractPruneRequest,
    ContractPruneResponse,
    ContractRenewedRequest,
    ContractRenewRequest,
    ContractRootsResponse,
    ContractsArchiveRequest,
    ContractSetUpdateRequest,
    ContractSize,
    ContractSpendingRecord,
    ContractsPrunableDataResponse,
    CopyObjectsRequest,
    Currency,
    FileContractID,
    GougingParams,
    Hash256,
    Host,
    HostAddress,
    HostCheck,
    HostsPriceTablesRequest,
    HostsRemoveRequest,
    HostsScanRequest,
    MigrationSlabsRequest,
    MultipartAbortRequest,
    MultipartAddPartRequest,
    MultipartCompleteRequest,
    MultipartCompleteResponse,
    MultipartCreateRequest,
    MultipartCreateResponse,
    MultipartListPartsRequest,
    MultipartListPartsResponse,
    MultipartListUploadsRequest,
    MultipartListUploadsResponse,
    MultipartUpload,
    Network,
    Object,
    ObjectMetadata,
    ObjectsListRequest,
    ObjectsListResponse,
    ObjectsRenameRequest,
    ObjectsStatsResponse,
    PackedSlab,
    PackedSlabsRequestGET,
    PackedSlabsRequestPOST,
    PublicKey,
    SearchHostsRequest,
    SiacoinElement,
    Slab,
    SlabBuffer,
    Transaction,
    TransactionID,
    UnhealthySlabsResponse,
    UpdateAllowlistRequest,
    UpdateBlocklistRequest,
    UploadParams,
    UploadSectorRequest,
    WalletFundRequest,
    WalletFundResponse,
    WalletRedistributeRequest,
    WalletResponse,
    WalletSendRequest,
    WalletSignRequest,
    Webhook,
    WebhookEvent,
    WebhookResponse,
)
from siaql.graphql.resolvers.filter import FilterInput, PaginationInput, SortInput

from strawberry.scalars import JSON


@strawberry.type
class BusQueries:
    @strawberry.field
    async def renterd_accounts(
        self,
        info: Info,
        owner: Optional[str] = None,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[Account]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_accounts",
            owner=owner,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_alerts(
        self,
        info: Info,
        opts: AlertsOpts.Input,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> AlertsResponse:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_alerts",
            opts=opts,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_autopilot(
        self,
        info: Info,
        id: str,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Autopilot:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_autopilot",
            id=id,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_get_state(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> BusStateResponse:
        """Get the current bus state"""
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_state",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_contract_renewed(
        self,
        info: Info,
        id: FileContractID,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> ContractMetadata:
        """Get the renewed contract for a given contract ID"""
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_contract_renewed",
            id=id,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_consensus_network(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Network:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_consensus_network",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_multipart_list_parts(
        self,
        info: Info,
        req: MultipartListPartsRequest.Input,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> MultipartListPartsResponse:
        """List parts of a multipart upload"""
        return await RenterdBaseResolver.handle_api_call(
            info,
            "list_multipart_parts",
            req=req,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_multipart_list_uploads(
        self,
        info: Info,
        req: MultipartListUploadsRequest.Input,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> MultipartListUploadsResponse:
        """List all multipart uploads"""
        return await RenterdBaseResolver.handle_api_call(
            info,
            "list_multipart_uploads",
            req=req,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_multipart_upload(
        self,
        info: Info,
        id: str,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> MultipartUpload:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_multipart_upload",
            id=id,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_autopilots(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[Autopilot]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_autopilots",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_buckets(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[Bucket]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_buckets",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_bucket(
        self,
        info: Info,
        name: str,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Bucket:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_bucket",
            name=name,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_consensus_state(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> ConsensusState:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_consensus_state",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_contracts(
        self,
        info: Info,
        contract_set: Optional[str] = None,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[ContractMetadata]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_contracts",
            contract_set=contract_set,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_contract(
        self,
        info: Info,
        id: FileContractID,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> ContractMetadata:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_contract",
            id=id,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_contract_size(
        self,
        info: Info,
        id: FileContractID,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> ContractSize:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_contract_size",
            id=id,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_get_hosts(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[Host]:
        """Get all hosts"""
        return await RenterdBaseResolver.handle_api_call(
            info, "get_hosts", filter_input=filter, sort_input=sort, pagination_input=pagination
        )

    @strawberry.field
    async def renterd_hosts_allowlist(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[PublicKey]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_hosts_allowlist",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_hosts_blocklist(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[str]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_hosts_blocklist",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_host(
        self,
        info: Info,
        public_key: PublicKey,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Host:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_host",
            public_key=public_key,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_search_hosts(
        self,
        info: Info,
        req: SearchHostsRequest.Input,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[Host]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "search_hosts",
            req=req,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_object(
        self,
        info: Info,
        path: str,
        bucket: Optional[str] = None,
        only_metadata: bool = False,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Object:
        return await RenterdBaseResolver.handle_api_call(
            info, "get_object", path=path, bucket=bucket, only_metadata=only_metadata
        )

    @strawberry.field
    async def renterd_search_objects(
        self,
        info: Info,
        key: str,
        bucket: str = "default",
        offset: int = 0,
        limit: int = -1,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[ObjectMetadata]:
        return await RenterdBaseResolver.handle_api_call(
            info, "search_objects", key=key, bucket=bucket, offset=offset, limit=limit
        )

    @strawberry.field
    async def renterd_objects_stats(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> ObjectsStatsResponse:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_objects_stats",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_slab_buffers(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[SlabBuffer]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_slab_buffers",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_wallet(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> WalletResponse:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_wallet",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_webhooks(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> WebhookResponse:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_webhooks",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_gouging_params(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> GougingParams:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_gouging_params",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_upload_params(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> UploadParams:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_upload_params",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_consensus_siafund_fee(
        self,
        info: Info,
        payout: Currency,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Currency:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_consensus_siafund_fee",
            payout=payout,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_contract_sets(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[str]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_contract_sets",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_contract_roots(
        self,
        info: Info,
        id: FileContractID,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> ContractRootsResponse:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_contract_roots",
            id=id,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_contract_ancestors(
        self,
        info: Info,
        id: FileContractID,
        min_start_height: int,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[ArchivedContract]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_contract_ancestors",
            id=id,
            min_start_height=min_start_height,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_contracts_prunable(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> ContractsPrunableDataResponse:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_contracts_prunable",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_hosts_scanning(
        self,
        info: Info,
        last_scan: Optional[str] = None,
        offset: int = 0,
        limit: int = -1,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[HostAddress]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_hosts_scanning",
            last_scan=last_scan,
            offset=offset,
            limit=limit,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_metric(
        self,
        info: Info,
        key: str,
        start: str,
        n: int,
        interval: str,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> JSON:  # Response type varies based on metric type
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_metric",
            key=key,
            start=start,
            n=n,
            interval=interval,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_settings(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[str]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_settings",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_setting(
        self,
        info: Info,
        key: str,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> str:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_setting",
            key=key,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_slab(
        self,
        info: Info,
        key: str,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Slab:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_slab",
            key=key,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_slab_objects(
        self,
        info: Info,
        key: str,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[ObjectMetadata]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_slab_objects",
            key=key,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_slabs_partial(
        self,
        info: Info,
        key: str,
        offset: int,
        length: int,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> str:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_slabs_partial",
            key=key,
            offset=offset,
            length=length,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_syncer_address(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> str:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_syncer_address",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_syncer_peers(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[str]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_syncer_peers",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_txpool_recommended_fee(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Currency:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_txpool_recommended_fee",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_txpool_transactions(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[Transaction]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_txpool_transactions",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_wallet_outputs(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[SiacoinElement]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_wallet_outputs",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_wallet_pending(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[Transaction]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_wallet_pending",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def renterd_wallet_transactions(
        self,
        info: Info,
        offset: int = 0,
        limit: int = -1,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[Transaction]:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "get_wallet_transactions",
            offset=offset,
            limit=limit,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )


@strawberry.type
class BusMutations:
    @strawberry.mutation
    async def renterd_update_autopilot_host_check(
        self, info: Info, autopilot_id: str, host_key: PublicKey, check: HostCheck.Input
    ) -> bool:
        await RenterdBaseResolver.handle_api_call(
            info, "update_autopilot_host_check", autopilot_id=autopilot_id, host_key=host_key, check=check
        )
        return True

    @strawberry.mutation
    async def renterd_fund_account(self, info: Info, req: AccountsFundRequest.Input) -> AccountsFundResponse:
        return await RenterdBaseResolver.handle_api_call(info, "fund_account", req=req)

    @strawberry.mutation
    async def renterd_save_accounts(self, info: Info, req: AccountsSaveRequest.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "save_accounts", req=req)
        return True

    @strawberry.mutation
    async def renterd_dismiss_alerts(self, info: Info, ids: List[Hash256]) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "dismiss_alerts", ids=ids)
        return True

    @strawberry.mutation
    async def renterd_register_alert(self, info: Info, alert: Alert.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "register_alert", alert=alert)
        return True

    @strawberry.mutation
    async def renterd_update_autopilot(self, info: Info, id: str, autopilot: Autopilot.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "update_autopilot", id=id, autopilot=autopilot)
        return True

    @strawberry.mutation
    async def renterd_create_bucket(self, info: Info, req: BucketCreateRequest.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "create_bucket", req=req)
        return True

    @strawberry.mutation
    async def renterd_delete_bucket(self, info: Info, name: str) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "delete_bucket", name=name)
        return True

    @strawberry.mutation
    async def renterd_update_contract_set(self, info: Info, set_name: str, req: ContractSetUpdateRequest.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "update_contract_set", set_name=set_name, req=req)
        return True

    @strawberry.mutation
    async def renterd_delete_contract_set(self, info: Info, set_name: str) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "delete_contract_set", set_name=set_name)
        return True

    @strawberry.mutation
    async def renterd_acquire_contract(
        self, info: Info, id: FileContractID, req: ContractAcquireRequest.Input
    ) -> ContractAcquireResponse:
        return await RenterdBaseResolver.handle_api_call(info, "acquire_contract", id=id, req=req)

    @strawberry.mutation
    async def renterd_keepalive_contract(self, info: Info, id: FileContractID, req: ContractKeepaliveRequest.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "keepalive_contract", id=id, req=req)
        return True

    @strawberry.mutation
    async def renterd_release_contract(self, info: Info, id: FileContractID, lock_id: int) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "release_contract", id=id, lock_id=lock_id)
        return True

    @strawberry.mutation
    async def renterd_prune_contract(
        self, info: Info, id: FileContractID, req: ContractPruneRequest.Input
    ) -> ContractPruneResponse:
        return await RenterdBaseResolver.handle_api_call(info, "prune_contract", id=id, req=req)

    @strawberry.mutation
    async def renterd_renew_contract(self, info: Info, id: FileContractID, req: ContractRenewRequest.Input) -> ContractMetadata:
        return await RenterdBaseResolver.handle_api_call(info, "renew_contract", id=id, req=req)

    @strawberry.mutation
    async def renterd_add_renewed_contract(
        self, info: Info, id: FileContractID, req: ContractRenewedRequest.Input
    ) -> ContractMetadata:
        return await RenterdBaseResolver.handle_api_call(info, "add_renewed_contract", id=id, req=req)

    @strawberry.mutation
    async def renterd_add_contract(self, info: Info, id: FileContractID, req: ContractAddRequest.Input) -> ContractMetadata:
        return await RenterdBaseResolver.handle_api_call(info, "add_contract", id=id, req=req)

    @strawberry.mutation
    async def renterd_archive_contracts(self, info: Info, req: ContractsArchiveRequest.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "archive_contracts", req=req)
        return True

    @strawberry.mutation
    async def renterd_record_contract_spending(self, info: Info, records: List[ContractSpendingRecord.Input]) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "record_contract_spending", records=records)
        return True

    @strawberry.mutation
    async def renterd_update_hosts_allowlist(self, info: Info, req: UpdateAllowlistRequest.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "update_hosts_allowlist", req=req)
        return True

    @strawberry.mutation
    async def renterd_delete_contract(self, info: Info, id: FileContractID) -> bool:
        """Delete a contract by ID"""
        await RenterdBaseResolver.handle_api_call(info, "delete_contract", id=id)
        return True

    @strawberry.mutation
    async def renterd_update_hosts_blocklist(self, info: Info, req: UpdateBlocklistRequest.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "update_hosts_blocklist", req=req)
        return True

    @strawberry.mutation
    async def renterd_record_hosts_scan(self, info: Info, req: HostsScanRequest.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "record_hosts_scan", req=req)
        return True

    @strawberry.mutation
    async def renterd_record_price_tables(self, info: Info, req: HostsPriceTablesRequest.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "record_price_tables", req=req)
        return True

    @strawberry.mutation
    async def renterd_add_object(self, info: Info, path: str, req: AddObjectRequest.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "add_object", path=path, req=req)
        return True

    @strawberry.mutation
    async def renterd_copy_object(self, info: Info, req: CopyObjectsRequest.Input) -> ObjectMetadata:
        return await RenterdBaseResolver.handle_api_call(info, "copy_object", req=req)

    @strawberry.mutation
    async def renterd_delete_object(self, info: Info, path: str, bucket: Optional[str] = None, batch: bool = False) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "delete_object", path=path, bucket=bucket, batch=batch)
        return True

    @strawberry.mutation
    async def renterd_rename_object(self, info: Info, req: ObjectsRenameRequest.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "rename_object", req=req)
        return True

    @strawberry.mutation
    async def renterd_list_objects(self, info: Info, req: ObjectsListRequest.Input) -> ObjectsListResponse:
        return await RenterdBaseResolver.handle_api_call(info, "list_objects", req=req)

    @strawberry.mutation
    async def renterd_create_multipart_upload(self, info: Info, req: MultipartCreateRequest.Input) -> MultipartCreateResponse:
        return await RenterdBaseResolver.handle_api_call(info, "create_multipart_upload", req=req)

    @strawberry.mutation
    async def renterd_abort_multipart_upload(self, info: Info, req: MultipartAbortRequest.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "abort_multipart_upload", req=req)
        return True

    @strawberry.mutation
    async def renterd_complete_multipart_upload(
        self, info: Info, req: MultipartCompleteRequest.Input
    ) -> MultipartCompleteResponse:
        return await RenterdBaseResolver.handle_api_call(info, "complete_multipart_upload", req=req)

    @strawberry.mutation
    async def renterd_add_multipart_part(self, info: Info, req: MultipartAddPartRequest.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "add_multipart_part", req=req)
        return True

    @strawberry.mutation
    async def renterd_form_contract(self, info: Info, req: ContractFormRequest.Input) -> ContractMetadata:
        return await RenterdBaseResolver.handle_api_call(info, "form_contract", req=req)

    @strawberry.mutation
    async def renterd_fetch_packed_slabs(self, info: Info, req: PackedSlabsRequestGET.Input) -> List[PackedSlab]:
        return await RenterdBaseResolver.handle_api_call(info, "fetch_packed_slabs", req=req)

    @strawberry.mutation
    async def renterd_mark_packed_slabs_uploaded(self, info: Info, req: PackedSlabsRequestPOST.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "mark_packed_slabs_uploaded", req=req)
        return True

    @strawberry.mutation
    async def renterd_delete_host_sector(self, info: Info, host_key: PublicKey, root: Hash256) -> int:
        return await RenterdBaseResolver.handle_api_call(info, "delete_host_sector", host_key=host_key, root=root)

    @strawberry.mutation
    async def renterd_wallet_fund(self, info: Info, req: WalletFundRequest.Input) -> WalletFundResponse:
        return await RenterdBaseResolver.handle_api_call(info, "wallet_fund", req=req)

    @strawberry.mutation
    async def renterd_wallet_redistribute(self, info: Info, req: WalletRedistributeRequest.Input) -> List[FileContractID]:
        return await RenterdBaseResolver.handle_api_call(info, "wallet_redistribute", req=req)

    @strawberry.mutation
    async def renterd_wallet_send_siacoins(self, info: Info, req: WalletSendRequest.Input) -> TransactionID:
        return await RenterdBaseResolver.handle_api_call(info, "wallet_send_siacoins", req=req)

    @strawberry.mutation
    async def renterd_wallet_sign_transaction(self, info: Info, req: WalletSignRequest.Input) -> Transaction:
        return await RenterdBaseResolver.handle_api_call(info, "wallet_sign_transaction", req=req)

    @strawberry.mutation
    async def renterd_wallet_discard_transaction(self, info: Info, transaction: Transaction.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "wallet_discard_transaction", transaction=transaction)
        return True

    @strawberry.mutation
    async def renterd_register_webhook(self, info: Info, webhook: Webhook.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "register_webhook", webhook=webhook)
        return True

    @strawberry.mutation
    async def renterd_delete_webhook(self, info: Info, webhook: Webhook.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "delete_webhook", webhook=webhook)
        return True

    @strawberry.mutation
    async def renterd_broadcast_action(self, info: Info, event: WebhookEvent.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "broadcast_action", event=event)
        return True

    @strawberry.mutation
    async def renterd_refresh_health(self, info: Info) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "refresh_health")
        return True

    @strawberry.mutation
    async def renterd_update_setting(self, info: Info, key: str, value: str) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "update_setting", key=key, value=value)
        return True

    @strawberry.mutation
    async def renterd_delete_setting(self, info: Info, key: str) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "delete_setting", key=key)
        return True

    @strawberry.mutation
    async def renterd_sync_connect(self, info: Info, addr: str) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "sync_connect", addr=addr)
        return True

    @strawberry.mutation
    async def renterd_txpool_broadcast(self, info: Info, transactions: List[Transaction.Input]) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "txpool_broadcast", transactions=transactions)
        return True

    @strawberry.mutation
    async def renterd_consensus_accept_block(self, info: Info, block: Block.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "consensus_accept_block", block=block)
        return True

    @strawberry.mutation
    async def renterd_update_bucket_policy(self, info: Info, name: str, policy: BucketPolicy.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "update_bucket_policy", name=name, policy=policy)
        return True

    @strawberry.mutation
    async def renterd_delete_contracts_all(self, info: Info) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "delete_contracts_all")
        return True

    @strawberry.mutation
    async def renterd_contract_broadcast(self, info: Info, id: FileContractID) -> TransactionID:
        return await RenterdBaseResolver.handle_api_call(info, "contract_broadcast", id=id)

    @strawberry.mutation
    async def renterd_hosts_reset_lost_sectors(self, info: Info, hostkey: PublicKey) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "hosts_reset_lost_sectors", hostkey=hostkey)
        return True

    @strawberry.mutation
    async def renterd_hosts_remove(self, info: Info, req: HostsRemoveRequest.Input) -> int:
        return await RenterdBaseResolver.handle_api_call(
            info,
            "hosts_remove",
            req=req,
        )

    @strawberry.mutation
    async def renterd_add_slabs_partial(
        self, info: Info, data: str, min_shards: int, total_shards: int, contract_set: str
    ) -> AddPartialSlabResponse:
        """Add partial slab data"""
        return await RenterdBaseResolver.handle_api_call(
            info,
            "add_slabs_partial",
            data=data,
            min_shards=min_shards,
            total_shards=total_shards,
            contract_set=contract_set,
        )

    @strawberry.mutation
    async def renterd_update_metric(self, info: Info, key: str, data: JSON) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "update_metric", key=key, data=data)
        return True

    @strawberry.mutation
    async def renterd_delete_metric(self, info: Info, key: str, cutoff: str) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "delete_metric", key=key, cutoff=cutoff)
        return True

    @strawberry.mutation
    async def renterd_slabs_migration(self, info: Info, req: MigrationSlabsRequest.Input) -> UnhealthySlabsResponse:
        return await RenterdBaseResolver.handle_api_call(info, "slabs_migration", req=req)

    @strawberry.mutation
    async def renterd_update_slab(self, info: Info, slab: Slab.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "update_slab", slab=slab)
        return True

    @strawberry.mutation
    async def renterd_upload_track(self, info: Info, id: str) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "upload_track", id=id)
        return True

    @strawberry.mutation
    async def renterd_upload_add_sector(self, info: Info, id: str, req: UploadSectorRequest.Input) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "upload_add_sector", req=req)
        return True

    @strawberry.mutation
    async def renterd_upload_finished(self, info: Info, id: str) -> bool:
        await RenterdBaseResolver.handle_api_call(info, "upload_finished", id=id)
        return True



# siaql/siaql/graphql/schemas/consensus.py
from typing import List, Optional, Union
from datetime import datetime
import strawberry
from strawberry.types import Info
from siaql.graphql.resolvers.walletd import WalletdBaseResolver
from siaql.graphql.schemas.types import (
    ChainIndex,
    ConsensusState,
    ConsensusUpdatesResponse,  # From consensus.State
    Network,  # From consensus.Network
    ApplyUpdate,  # From consensus.ApplyUpdate
    RevertUpdate,  # From consensus.RevertUpdate
)
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput


@strawberry.type
class ConsensusQueries(WalletdBaseResolver):
    @strawberry.field
    async def walletd_consensus_network(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Network:
        """Get the consensus network information"""
        return await WalletdBaseResolver.handle_api_call(
            info, "get_consensus_network", filter_input=filter, sort_input=sort, pagination_input=pagination
        )

    @strawberry.field
    async def walletd_consensus_tip(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> ChainIndex:
        """Get the current consensus tip"""
        return await WalletdBaseResolver.handle_api_call(
            info, "get_consensus_tip", filter_input=filter, sort_input=sort, pagination_input=pagination
        )

    @strawberry.field
    async def walletd_consensus_tip_state(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> ConsensusState:
        """Get the current consensus tip state"""
        return await WalletdBaseResolver.handle_api_call(
            info, "get_consensus_tip_state", filter_input=filter, sort_input=sort, pagination_input=pagination
        )

    @strawberry.field
    async def walletd_consensus_index(
        self,
        info: Info,
        height: int,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> ChainIndex:
        """Get consensus index at specified height"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_consensus_index",
            height=height,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def walletd_consensus_updates(
        self,
        info: Info,
        index: ChainIndex.Input,
        limit: int = 10,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> ConsensusUpdatesResponse:
        """Get consensus updates since specified index"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_consensus_updates",
            index=index,
            limit=limit,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )



# siaql/siaql/api/walletd.py
from typing import Dict, List, Optional, Any, Union
import httpx
from datetime import datetime
from siaql.api.utils import handle_api_errors, APIError
from siaql.graphql.schemas.types import (
    Address,
    ApplyUpdate,
    Balance,
    ConsensusUpdatesResponse,
    Currency,
    GatewayPeer,
    RevertUpdate,
    Transaction,
    TxpoolBroadcastRequest,
    TxpoolTransactionsResponse,
    V2Transaction,
    ChainIndex,
    Block,
    SiacoinElement,
    SiafundElement,
    StateResponse,
    RescanResponse,
    ConsensusState,
    Network,
    Wallet,
    WalletEvent,
    WalletFundRequest,
    WalletFundResponse,
    WalletFundSFRequest,
    WalletReleaseRequest,
    WalletReserveRequest,
    WalletUpdateRequest,
)


class WalletdError(APIError):
    """Specific exception for Walletd API errors"""

    pass


class WalletdClient:
    def __init__(self, base_url: str, api_password: Optional[str] = None):
        # Ensure base_url doesn't have trailing slash and has /api
        self.base_url = f"{base_url.rstrip('/')}/api"
        if api_password:
            auth = httpx.BasicAuth(username="", password=api_password)
            self.client = httpx.AsyncClient(base_url=self.base_url, auth=auth, timeout=30.0)
        else:
            self.client = httpx.AsyncClient(base_url=self.base_url, timeout=30.0)

    async def close(self):
        """Close the HTTP client"""
        await self.client.aclose()

    # State endpoints
    @handle_api_errors(WalletdError)
    async def get_state(self) -> StateResponse:
        """Get the current state of the walletd daemon"""
        response = await self.client.get("/state")
        response.raise_for_status()
        return response.json()

    # Consensus endpoints
    @handle_api_errors(WalletdError)
    async def get_consensus_network(self) -> Network:
        """Get consensus network parameters"""
        response = await self.client.get("/consensus/network")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_consensus_tip(self) -> ChainIndex:
        """Get current consensus tip"""
        response = await self.client.get("/consensus/tip")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_consensus_tip_state(self) -> ConsensusState:
        """Get current consensus tip state"""
        response = await self.client.get("/consensus/tipstate")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_consensus_index(self, height: int) -> ChainIndex:
        """Get consensus index at specified height"""
        response = await self.client.get(f"/consensus/index/{height}")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_consensus_updates(self, index: ChainIndex, limit: int) -> ConsensusUpdatesResponse:
        """Get consensus updates since index."""
        response = await self.client.get(f"/consensus/updates/{index}", params={"limit": limit})
        return response.json()

    # Syncer endpoints
    @handle_api_errors(WalletdError)
    async def get_syncer_peers(self) -> List[GatewayPeer]:
        """Get list of connected peers"""
        response = await self.client.get("/syncer/peers")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def post_syncer_connect(self, addr: str) -> None:
        """Connect to a peer"""
        response = await self.client.post("/syncer/connect", json=addr)
        response.raise_for_status()

    @handle_api_errors(WalletdError)
    async def post_syncer_broadcast_block(self, block: Block) -> None:
        """Broadcast a block to all peers"""
        response = await self.client.post("/syncer/broadcast/block", json=block)
        response.raise_for_status()

    # Transaction Pool endpoints

    @handle_api_errors(WalletdError)
    async def get_txpool_parents(self, txn: Transaction) -> List[Transaction]:
        """Get parent transactions from pool"""
        response = await self.client.post("/txpool/parents", json=txn)
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_txpool_transactions(self) -> TxpoolTransactionsResponse:
        """Get all transactions in the transaction pool"""
        response = await self.client.get("/txpool/transactions")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_txpool_fee(self) -> Currency:
        """Get the recommended transaction fee"""
        response = await self.client.get("/txpool/fee")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def txpool_broadcast(self, req: TxpoolBroadcastRequest) -> None:
        """Broadcast transactions"""
        await self.client.post("/txpool/broadcast", json=req.dict())

    # Wallet endpoints

    @handle_api_errors(WalletdError)
    async def get_wallets(self) -> List[Wallet]:
        """Get all wallets"""
        response = await self.client.get("/wallets")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def post_add_wallet(self, wallet_update: WalletUpdateRequest) -> Wallet:
        """Add a new wallet"""
        response = await self.client.post("/wallets", json=wallet_update)
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def post_update_wallet(self, wallet_id: str, wallet_update: WalletUpdateRequest) -> Wallet:
        """Update a wallet"""
        response = await self.client.post(f"/wallets/{wallet_id}", json=wallet_update)
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def delete_wallet(self, wallet_id: str) -> None:
        """Delete a wallet"""
        response = await self.client.delete(f"/wallets/{wallet_id}")
        response.raise_for_status()

    @handle_api_errors(WalletdError)
    async def get_wallet_addresses(self, wallet_id: str) -> List[Address]:
        """Get addresses for a wallet"""
        response = await self.client.get(f"/wallets/{wallet_id}/addresses")
        response.raise_for_status()
        return response.json()

    # Wallet-specific operations
    @handle_api_errors(WalletdError)
    async def add_wallet_address(self, wallet_id: str, address: Address) -> None:
        """Add an address to a wallet"""
        response = await self.client.put(f"/wallets/{wallet_id}/addresses", json=address)
        response.raise_for_status()

    @handle_api_errors(WalletdError)
    async def delete_wallet_address(self, wallet_id: str, address: str) -> None:
        """Remove an address from a wallet"""
        response = await self.client.delete(f"/wallets/{wallet_id}/addresses/{address}")
        response.raise_for_status()

    @handle_api_errors(WalletdError)
    async def get_wallet_balance(self, wallet_id: str) -> Balance:
        """Get wallet balance"""
        response = await self.client.get(f"/wallets/{wallet_id}/balance")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_wallet_events(self, wallet_id: str, offset: int = 0, limit: int = 500) -> List[WalletEvent]:
        """Get wallet events"""
        response = await self.client.get(f"/wallets/{wallet_id}/events", params={"offset": offset, "limit": limit})
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_wallet_unconfirmed_events(self, wallet_id: str) -> List[WalletEvent]:
        """Get unconfirmed wallet events"""
        response = await self.client.get(f"/wallets/{wallet_id}/events/unconfirmed")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_wallet_siacoin_outputs(
        self, wallet_id: str, offset: int = 0, limit: int = 1000
    ) -> List[SiacoinElement]:
        """Get wallet siacoin outputs"""
        response = await self.client.get(
            f"/wallets/{wallet_id}/outputs/siacoin", params={"offset": offset, "limit": limit}
        )
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_wallet_siafund_outputs(
        self, wallet_id: str, offset: int = 0, limit: int = 1000
    ) -> List[SiafundElement]:
        """Get wallet siafund outputs"""
        response = await self.client.get(
            f"/wallets/{wallet_id}/outputs/siafund", params={"offset": offset, "limit": limit}
        )
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def post_wallet_reserve(self, wallet_id: str, reserve_request: WalletReserveRequest) -> None:
        """Reserve outputs"""
        response = await self.client.post(f"/wallets/{wallet_id}/reserve", json=reserve_request)
        response.raise_for_status()

    @handle_api_errors(WalletdError)
    async def post_wallet_release(self, wallet_id: str, release_request: WalletReleaseRequest) -> None:
        """Release outputs"""
        response = await self.client.post(f"/wallets/{wallet_id}/release", json=release_request)
        response.raise_for_status()

    @handle_api_errors(WalletdError)
    async def post_wallet_fund(self, wallet_id: str, fund_request: WalletFundRequest) -> WalletFundResponse:
        """Fund a transaction"""
        response = await self.client.post(f"/wallets/{wallet_id}/fund", json=fund_request)
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def post_wallet_fund_siafund(self, wallet_id: str, fund_request: WalletFundSFRequest) -> WalletFundResponse:
        """Fund a siafund transaction"""
        response = await self.client.post(f"/wallets/{wallet_id}/fundsf", json=fund_request)
        response.raise_for_status()
        return response.json()

    # Address-related endpoints

    @handle_api_errors(WalletdError)
    async def get_address_balance(self, address: str) -> Balance:
        """Get balance for address"""
        response = await self.client.get(f"/addresses/{address}/balance")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_address_events(self, address: str, offset: int = 0, limit: int = 500) -> List[WalletEvent]:
        """Get events for an address"""
        response = await self.client.get(f"/addresses/{address}/events", params={"offset": offset, "limit": limit})
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_address_unconfirmed_events(self, address: str) -> List[WalletEvent]:
        """Get unconfirmed events for an address"""
        response = await self.client.get(f"/addresses/{address}/events/unconfirmed")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_address_siacoin_outputs(
        self, address: str, offset: int = 0, limit: int = 1000
    ) -> List[SiacoinElement]:
        """Get siacoin outputs for an address"""
        response = await self.client.get(
            f"/addresses/{address}/outputs/siacoin", params={"offset": offset, "limit": limit}
        )
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_address_siafund_outputs(
        self, address: str, offset: int = 0, limit: int = 1000
    ) -> List[SiafundElement]:
        """Get siafund outputs for an address"""
        response = await self.client.get(
            f"/addresses/{address}/outputs/siafund", params={"offset": offset, "limit": limit}
        )
        response.raise_for_status()
        return response.json()

    # Event-related endpoints
    @handle_api_errors(WalletdError)
    async def get_event(self, event_id: str) -> WalletEvent:
        """Get a specific event"""
        response = await self.client.get(f"/events/{event_id}")
        response.raise_for_status()
        return response.json()

    # Rescan endpoints
    @handle_api_errors(WalletdError)
    async def get_rescan_status(self) -> RescanResponse:
        """Get rescan status"""
        response = await self.client.get("/rescan")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(WalletdError)
    async def start_rescan(self, height: int) -> None:
        """Start rescan from height"""
        response = await self.client.post("/rescan", json=height)
        response.raise_for_status()

    # Output endpoints
    @handle_api_errors(WalletdError)
    async def get_siacoin_output(self, id: str) -> SiacoinElement:
        """Get siacoin output"""
        response = await self.client.get(f"/outputs/siacoin/{id}")
        return response.json()

    @handle_api_errors(WalletdError)
    async def get_siafund_output(self, id: str) -> SiafundElement:
        """Get siafund output"""
        response = await self.client.get(f"/outputs/siafund/{id}")
        return response.json()



import strawberry
from typing import List, Optional
from strawberry.types import Info

from siaql.graphql.schemas.types import (
    Contract,
    ContractFilter,
    FileContractID,
    HostdContractsResponse,
    IntegrityCheckResult,
)
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput

from siaql.graphql.resolvers.hostd import HostdBaseResolver


@strawberry.type
class ContractQueries:
    @strawberry.field
    async def hostd_contracts(
        self,
        info: Info,
        filter: ContractFilter.Input,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> HostdContractsResponse:
        """Get contracts matching the filter"""
        return await HostdBaseResolver.handle_api_call(
            info, "post_contracts", filter=filter, sort_input=sort, pagination_input=pagination
        )

    @strawberry.field
    async def hostd_contract(
        self,
        info: Info,
        id: FileContractID,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Contract:
        """Get a specific contract by ID"""
        return await HostdBaseResolver.handle_api_call(
            info, "get_contract", id=id, filter_input=filter, sort_input=sort, pagination_input=pagination
        )

    @strawberry.field
    async def hostd_contract_integrity(
        self,
        info: Info,
        id: FileContractID,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Optional[IntegrityCheckResult]:
        """Get integrity check result for a contract"""
        return await HostdBaseResolver.handle_api_call(
            info, "get_contract_integrity", id=id, filter_input=filter, sort_input=sort, pagination_input=pagination
        )


@strawberry.type
class ContractMutations:
    @strawberry.mutation
    async def hostd_check_contract_integrity(self, info: Info, id: FileContractID) -> bool:
        """Start integrity check for a contract"""
        await HostdBaseResolver.handle_api_call(info, "put_contract_integrity", id=id)
        return True

    @strawberry.mutation
    async def hostd_delete_contract_integrity(self, info: Info, id: FileContractID) -> bool:
        """Delete integrity check result for a contract"""
        await HostdBaseResolver.handle_api_call(info, "delete_contract_integrity", id=id)
        return True



# siaql/graphql/schemas/walletd/txpool.py
from typing import List, Optional, Dict, Any
import strawberry
from strawberry.types import Info
from siaql.graphql.resolvers.walletd import WalletdBaseResolver
from strawberry.scalars import JSON
from siaql.graphql.schemas.types import (
    Transaction,
    TxpoolBroadcastRequest,
    V2Transaction,
    ChainIndex,
    Currency,
    TxpoolTransactionsResponse,
)
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput


@strawberry.type
class TxpoolQueries:
    @strawberry.field
    async def walletd_txpool_parents(
        self,
        info: Info,
        transaction: Transaction.Input,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[Transaction]:
        """Get parent transactions from pool"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_txpool_parents",
            transaction=transaction,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def walletd_txpool_transactions(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> TxpoolTransactionsResponse:
        """Get all transactions in the transaction pool"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_txpool_transactions",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def walletd_txpool_fee(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Currency:
        """Get the recommended transaction fee"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_txpool_fee",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )


@strawberry.type
class TxpoolMutations:
    @strawberry.mutation
    async def walletd_txpool_broadcast(self, info: Info, req: TxpoolBroadcastRequest.Input) -> bool:
        """Broadcast transactions to network"""
        await WalletdBaseResolver.handle_api_call(info, "txpool_broadcast", req=req)
        return True



import strawberry
from strawberry.types import Info

from siaql.graphql.schemas.types import SystemDirResponse
from siaql.graphql.resolvers.hostd import HostdBaseResolver
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput
from typing import Optional


@strawberry.type
class SystemQueries:
    @strawberry.field
    async def hostd_system_dir(
        self,
        info: Info,
        path: str,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> SystemDirResponse:
        """Get directory contents"""
        return await HostdBaseResolver.handle_api_call(
            info, "get_system_dir", path=path, filter_input=filter, sort_input=sort, pagination_input=pagination
        )


@strawberry.type
class SystemMutations:
    @strawberry.mutation
    async def hostd_create_dir(self, info: Info, path: str) -> bool:
        """Create a directory"""
        await HostdBaseResolver.handle_api_call(info, "put_system_dir", path=path)
        return True

    @strawberry.mutation
    async def hostd_backup_sqlite3(self, info: Info, path: str) -> bool:
        """Create a backup of the SQLite3 database"""
        await HostdBaseResolver.handle_api_call(info, "post_system_sqlite3_backup", path=path)
        return True



from typing import List, Optional, Dict
from strawberry.types import Info
import strawberry
from datetime import datetime
from siaql.graphql.resolvers.walletd import WalletdBaseResolver
from siaql.graphql.schemas.types import WalletEvent, SiacoinElement, SiafundElement, Balance
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput


@strawberry.type
class AddressQueries:
    @strawberry.field
    async def walletd_address_balance(
        self,
        info: Info,
        address: str,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> Balance:
        """Get balance for address"""
        data = await WalletdBaseResolver.handle_api_call(
            info,
            "get_address_balance",
            address=address,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )
        return data

    @strawberry.field
    async def walletd_address_events(
        self,
        info: Info,
        address: str,
        offset: int = 0,
        limit: int = 500,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[WalletEvent]:
        """Get events for an address"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_address_events",
            address=address,
            offset=offset,
            limit=limit,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def walletd_address_unconfirmed_events(
        self,
        info: Info,
        address: str,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[WalletEvent]:
        """Get unconfirmed events for an address"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_address_unconfirmed_events",
            address=address,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def walletd_address_siacoin_outputs(
        self,
        info: Info,
        address: str,
        offset: int = 0,
        limit: int = 1000,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[SiacoinElement]:
        """Get siacoin outputs for an address"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_address_siacoin_outputs",
            address=address,
            offset=offset,
            limit=limit,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )

    @strawberry.field
    async def walletd_address_siafund_outputs(
        self,
        info: Info,
        address: str,
        offset: int = 0,
        limit: int = 1000,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[SiafundElement]:
        """Get siafund outputs for an address"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_address_siafund_outputs",
            address=address,
            offset=offset,
            limit=limit,
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )



from typing import Any, Callable, Dict, Optional, TypeVar
from strawberry.types import Info
from siaql.graphql.resolvers.converter import TypeConverter
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput, QueryFiltering
from functools import wraps
import logging

T = TypeVar("T")

logger = logging.getLogger("siaql.resolvers.hostd")


class HostdBaseResolver:
    """Base resolver class for Hostd API"""

    @classmethod
    async def handle_api_call(
        cls,
        info: Info,
        method: str,
        transform_func: Optional[Callable[[Dict], T]] = None,
        filter_input: Optional[FilterInput] = None,
        sort_input: Optional[SortInput] = None,
        pagination_input: Optional[PaginationInput] = None,
        *args,
        **kwargs,
    ) -> Any:
        """Generic method to handle API calls with error handling"""
        # Check if endpoint is skipped
        if info.context["skipped_endpoints"].get("hostd", False):
            raise Exception("Hostd endpoint was skipped during configuration. This query is not available.")

        client = info.context["hostd_client"]
        method_func = getattr(client, method)
        try:
            # 1. Get raw data from API
            result = await method_func(*args, **kwargs)
            logger.debug("Executing method: %s", method_func)

            # 2. Apply any custom transformations
            if transform_func:
                result = transform_func(result)

            # 3. Convert to proper GraphQL types if this is a typed field
            if hasattr(info, "_field"):
                field_type = info._field.type
                # Convert the entire result to proper GraphQL types
                result = TypeConverter.convert(result, field_type)

            # 4. Apply filtering, sorting, and pagination AFTER type conversion
            if isinstance(result, list):
                if filter_input:
                    # Now the data is in proper GraphQL types, making it easier to filter
                    result = QueryFiltering.apply_filter(result, filter_input)
                if sort_input:
                    result = QueryFiltering.apply_sort(result, sort_input)
                if pagination_input:
                    result = QueryFiltering.apply_pagination(result, pagination_input)

            return result
        except Exception as e:
            logger.error("Error in handle_api_call: %s", e)
            raise e



from datetime import datetime
from typing import List, Optional, Dict, Any, Union
import httpx
from siaql.graphql.schemas.types import (
    AddVolumeRequest,
    Alert,
    ChainIndex,
    ConsensusState,
    Contract,
    ContractFilter,
    FileContractID,
    FundingSource,
    Hash256,
    HostdAccount,
    HostdContractsResponse,
    HostdState,
    IntegrityCheckResult,
    Metrics,
    MetricsInterval,
    Network,
    Peer,
    PinnedSettings,
    HostSettings,
    RegisterWebHookRequest,
    ResizeVolumeRequest,
    SystemDirResponse,
    TransactionID,
    UpdateVolumeRequest,
    VerifySectorResponse,
    Volume,
    VolumeMeta,
    WalletSendSiacoinsRequest,
    Webhook,
    WalletEvent,
    WalletResponse,
    Currency,
)
from siaql.api.utils import handle_api_errors


class HostdError(Exception):
    """Base exception for Hostd API errors"""

    pass


class HostdClient:
    """Client for interacting with the Hostd REST API"""

    def __init__(self, base_url: str, api_password: Optional[str] = None):
        # Ensure base_url doesn't have trailing slash and has /api
        self.base_url = f"{base_url.rstrip('/')}/api"
        if api_password:
            auth = httpx.BasicAuth(username="", password=api_password)
            self.client = httpx.AsyncClient(base_url=self.base_url, auth=auth, timeout=30.0)
        else:
            self.client = httpx.AsyncClient(base_url=self.base_url, timeout=30.0)

    async def close(self):
        """Close the HTTP client"""
        await self.client.aclose()

    # State endpoints
    @handle_api_errors(HostdError)
    async def get_state(self) -> HostdState:
        """Get the current state of the hostd daemon"""
        response = await self.client.get("/state")
        response.raise_for_status()
        return response.json()

    # Consensus endpoints
    @handle_api_errors(HostdError)
    async def get_consensus_tip(self) -> ChainIndex:
        """Get the current consensus tip"""
        response = await self.client.get("/consensus/tip")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def get_consensus_tip_state(self) -> ConsensusState:
        """Get the current consensus tip state"""
        response = await self.client.get("/consensus/tipstate")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def get_consensus_network(self) -> Network:
        """Get consensus network parameters"""
        response = await self.client.get("/consensus/network")
        response.raise_for_status()
        return response.json()

    # Syncer endpoints
    @handle_api_errors(HostdError)
    async def get_syncer_address(self) -> str:
        """Get syncer address"""
        response = await self.client.get("/syncer/address")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def get_syncer_peers(self) -> List[Peer]:
        """Get list of connected peers"""
        response = await self.client.get("/syncer/peers")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def put_syncer_peer(self, address: str) -> None:
        """Connect to a peer"""
        response = await self.client.put("/syncer/peers", json={"address": address})
        response.raise_for_status()

    # Index endpoints
    @handle_api_errors(HostdError)
    async def get_index_tip(self) -> ChainIndex:
        """Get current index tip"""
        response = await self.client.get("/index/tip")
        response.raise_for_status()
        return response.json()

    # Alert endpoints
    @handle_api_errors(HostdError)
    async def get_alerts(self) -> List[Alert]:
        """Get active alerts"""
        response = await self.client.get("/alerts")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def post_alerts_dismiss(self, ids: List[Hash256]) -> None:
        """Dismiss alerts"""
        response = await self.client.post("/alerts/dismiss", json=ids)
        response.raise_for_status()

    # Settings endpoints
    @handle_api_errors(HostdError)
    async def get_settings(self) -> HostSettings:
        """Get host settings"""
        response = await self.client.get("/settings")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def patch_settings(self, settings: HostSettings) -> HostSettings:
        """Update host settings"""
        response = await self.client.patch("/settings", json=settings)
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def post_announce(self) -> None:
        """Announce the host"""
        response = await self.client.post("/settings/announce")
        response.raise_for_status()

    @handle_api_errors(HostdError)
    async def put_ddns_update(self, force: bool = False) -> None:
        """Update dynamic DNS"""
        response = await self.client.put("/settings/ddns/update", params={"force": force})
        response.raise_for_status()

    @handle_api_errors(HostdError)
    async def get_pinned_settings(self) -> PinnedSettings:
        """Get pinned settings"""
        response = await self.client.get("/settings/pinned")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def put_pinned_settings(self, settings: PinnedSettings) -> None:
        """Update pinned settings"""
        response = await self.client.put("/settings/pinned", json=settings)
        response.raise_for_status()

    # Metrics endpoints
    @handle_api_errors(HostdError)
    async def get_metrics(self, timestamp: Optional[datetime] = None) -> Metrics:
        """Get metrics at specified timestamp"""
        params = {}
        if timestamp:
            params["timestamp"] = timestamp.isoformat()
        response = await self.client.get("/metrics", params=params)
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def get_period_metrics(self, start: datetime, periods: int, interval: MetricsInterval) -> List[Metrics]:
        """Get metrics for multiple periods"""
        params = {"start": start.isoformat(), "periods": str(periods)}
        response = await self.client.get(f"/metrics/{interval}", params=params)
        response.raise_for_status()
        return response.json()

    # Contract endpoints
    @handle_api_errors(HostdError)
    async def post_contracts(self, filter: ContractFilter) -> HostdContractsResponse:
        """Get contracts matching filter"""
        response = await self.client.post("/contracts", json=filter)
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def get_contract(self, id: FileContractID) -> Contract:
        """Get specific contract"""
        response = await self.client.get(f"/contracts/{id}")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def get_contract_integrity(self, id: FileContractID) -> IntegrityCheckResult:
        """Get contract integrity check result"""
        response = await self.client.get(f"/contracts/{id}/integrity")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def put_contract_integrity(self, id: FileContractID) -> None:
        """Start contract integrity check"""
        response = await self.client.put(f"/contracts/{id}/integrity")
        response.raise_for_status()

    @handle_api_errors(HostdError)
    async def delete_contract_integrity(self, id: FileContractID) -> None:
        """Delete contract integrity check result"""
        response = await self.client.delete(f"/contracts/{id}/integrity")
        response.raise_for_status()

    # Account endpoints
    @handle_api_errors(HostdError)
    async def get_accounts(self, limit: int = 100, offset: int = 0) -> List[HostdAccount]:
        """Get accounts with pagination"""
        params = {"limit": limit, "offset": offset}
        response = await self.client.get("/accounts", params=params)
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def get_account_funding(self, account: str) -> List[FundingSource]:
        """Get account funding sources"""
        response = await self.client.get(f"/accounts/{account}/funding")
        response.raise_for_status()
        return response.json()

    # Sector endpoints
    @handle_api_errors(HostdError)
    async def delete_sector(self, root: Hash256) -> None:
        """Delete a sector"""
        response = await self.client.delete(f"/sectors/{root}")
        response.raise_for_status()

    @handle_api_errors(HostdError)
    async def get_verify_sector(self, root: Hash256) -> VerifySectorResponse:
        """Verify a sector"""
        response = await self.client.get(f"/sectors/{root}/verify")
        response.raise_for_status()
        return response.json()

    # Volume endpoints
    @handle_api_errors(HostdError)
    async def get_volumes(self) -> List[VolumeMeta]:
        """Get all volumes"""
        response = await self.client.get("/volumes")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def post_volume(self, req: AddVolumeRequest) -> Volume:
        """Add a new volume"""
        response = await self.client.post("/volumes", json=req)
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def get_volume(self, id: int) -> VolumeMeta:
        """Get specific volume"""
        response = await self.client.get(f"/volumes/{id}")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def put_volume(self, id: int, req: UpdateVolumeRequest) -> None:
        """Update volume settings"""
        response = await self.client.put(f"/volumes/{id}", json=req)
        response.raise_for_status()

    @handle_api_errors(HostdError)
    async def delete_volume(self, id: int, force: bool = False) -> None:
        """Delete a volume"""
        params = {"force": force}
        response = await self.client.delete(f"/volumes/{id}", params=params)
        response.raise_for_status()

    @handle_api_errors(HostdError)
    async def put_volume_resize(self, id: int, req: ResizeVolumeRequest) -> None:
        """Resize a volume"""
        response = await self.client.put(f"/volumes/{id}/resize", json=req)
        response.raise_for_status()

    @handle_api_errors(HostdError)
    async def delete_volume_cancel_op(self, id: int) -> None:
        """Cancel ongoing volume operation"""
        response = await self.client.delete(f"/volumes/{id}/cancel")
        response.raise_for_status()

    # System endpoints
    @handle_api_errors(HostdError)
    async def get_system_dir(self, path: str) -> SystemDirResponse:
        """Get directory contents"""
        params = {"path": path}
        response = await self.client.get("/system/dir", params=params)
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def put_system_dir(self, path: str) -> None:
        """Create a directory"""
        data = {"path": path}
        response = await self.client.put("/system/dir", json=data)
        response.raise_for_status()

    @handle_api_errors(HostdError)
    async def post_system_sqlite3_backup(self, path: str) -> None:
        """Create SQLite3 backup"""
        data = {"path": path}
        response = await self.client.post("/system/sqlite3/backup", json=data)
        response.raise_for_status()

    # Wallet endpoints
    @handle_api_errors(HostdError)
    async def get_wallet(self) -> WalletResponse:
        """Get wallet state"""
        response = await self.client.get("/wallet")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def get_wallet_events(self, limit: int = 100, offset: int = 0) -> List[WalletEvent]:
        """Get wallet events with pagination"""
        params = {"limit": limit, "offset": offset}
        response = await self.client.get("/wallet/events", params=params)
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def get_wallet_pending(self) -> List[WalletEvent]:
        """Get pending wallet events"""
        response = await self.client.get("/wallet/pending")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def post_wallet_send(self, req: WalletSendSiacoinsRequest) -> TransactionID:
        """Send siacoins"""
        response = await self.client.post("/wallet/send", json=req)
        response.raise_for_status()
        return response.json()

    # TPool endpoints
    @handle_api_errors(HostdError)
    async def get_tpool_fee(self) -> Currency:
        """Get recommended transaction fee"""
        response = await self.client.get("/tpool/fee")
        response.raise_for_status()
        return response.json()

    # Webhook endpoints
    @handle_api_errors(HostdError)
    async def get_webhooks(self) -> List[Webhook]:
        """Get all webhooks"""
        response = await self.client.get("/webhooks")
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def post_webhooks(self, req: RegisterWebHookRequest) -> Webhook:
        """Register a new webhook"""
        response = await self.client.post("/webhooks", json=req)
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def put_webhooks(self, id: int, req: RegisterWebHookRequest) -> Webhook:
        """Update an existing webhook"""
        response = await self.client.put(f"/webhooks/{id}", json=req)
        response.raise_for_status()
        return response.json()

    @handle_api_errors(HostdError)
    async def post_webhooks_test(self, id: int) -> None:
        """Test a webhook"""
        response = await self.client.post(f"/webhooks/{id}/test")
        response.raise_for_status()

    @handle_api_errors(HostdError)
    async def delete_webhooks(self, id: int) -> None:
        """Delete a webhook"""
        response = await self.client.delete(f"/webhooks/{id}")
        response.raise_for_status()



# siaql/graphql/schemas/walletd/syncer.py
from typing import List, Optional
from datetime import datetime
import strawberry
from strawberry.types import Info
from siaql.graphql.resolvers.walletd import WalletdBaseResolver
from siaql.graphql.schemas.types import GatewayPeer, Block
from siaql.graphql.resolvers.filter import FilterInput, SortInput, PaginationInput


@strawberry.type
class SyncerQueries:
    @strawberry.field
    async def walletd_syncer_peers(
        self,
        info: Info,
        filter: Optional[FilterInput] = None,
        sort: Optional[SortInput] = None,
        pagination: Optional[PaginationInput] = None,
    ) -> List[GatewayPeer]:
        """Get list of connected peers"""
        return await WalletdBaseResolver.handle_api_call(
            info,
            "get_syncer_peers",
            filter_input=filter,
            sort_input=sort,
            pagination_input=pagination,
        )


@strawberry.type
class SyncerMutations:
    @strawberry.mutation
    async def walletd_syncer_connect(self, info: Info, addr: str) -> bool:
        """Connect to a peer"""
        await WalletdBaseResolver.handle_api_call(info, "post_syncer_connect", addr=addr)
        return True

    @strawberry.mutation
    async def walletd_syncer_broadcast_block(self, info: Info, block: Block.Input) -> bool:
        """Broadcast a block to all peers"""
        await WalletdBaseResolver.handle_api_call(info, "post_syncer_broadcast_block", block=block)
        return True
